"""Tools module tests."""

from __future__ import annotations

from url_normalize.tools import force_unicode


def test_force_unicode_with_bytes() -> None:
    """Test force_unicode handles bytes input correctly."""
    test_bytes = b"hello world"
    result = force_unicode(test_bytes)
    assert result == "hello world"



"""Tests for normalize_userinfo function."""

import pytest

from url_normalize.url_normalize import normalize_userinfo


@pytest.mark.parametrize(
    ("userinfo", "expected"),
    [
        (":@", ""),
        ("", ""),
        ("@", ""),
        ("user:password@", "user:password@"),
        ("user@", "user@"),
    ],
)
def test_normalize_userinfo_result_is_expected(userinfo: str, expected: str) -> None:
    """Assert we got expected results from the normalize_userinfo function."""
    result = normalize_userinfo(userinfo)
    assert result == expected, userinfo



"""Tests for normalize_host function."""

import pytest

from url_normalize.url_normalize import normalize_host


@pytest.mark.parametrize(
    ("host", "expected"),
    [
        # Basic cases
        ("site.com", "site.com"),
        ("SITE.COM", "site.com"),
        ("site.com.", "site.com"),
        # Cyrillic domains
        ("пример.испытание", "xn--e1afmkfd.xn--80akhbyknj4f"),
        # Mixed case with Cyrillic
        ("ExAmPle.РФ", "example.xn--p1ai"),
        # IDNA2008 with UTS46
        ("faß.de", "fass.de"),  # Normalize using transitional rules
        # Edge cases
        ("ドメイン.テスト", "xn--eckwd4c7c.xn--zckzah"),  # Japanese
        ("domain.café", "domain.xn--caf-dma"),  # Latin with diacritic
        # Normalization tests
        ("über.example", "xn--ber-goa.example"),  # IDNA 2008 for umlaut
        ("example。com", "example.com"),  # Normalize full-width punctuation
    ],
)
def test_normalize_host_result_is_expected(host: str, expected: str) -> None:
    """Assert we got expected results from the normalize_host function."""
    result = normalize_host(host)
    assert result == expected, host



"""Tests for normalize_port function."""

import pytest

from url_normalize.url_normalize import normalize_port


@pytest.mark.parametrize(
    ("port", "expected"),
    [
        ("8080", "8080"),  # Non-default port
        ("", ""),  # Empty port
        ("80", ""),  # Default HTTP port
        ("string", "string"),  # Non-numeric port (should pass through)
        # Add more cases as needed, e.g., for HTTPS
        pytest.param("443", "", id="https_default_port"),
    ],
)
def test_normalize_port_result_is_expected(port: str, expected: str):
    """Assert we got expected results from the normalize_port function."""
    # Test with 'http' scheme for most cases
    scheme = "https" if port == "443" else "http"

    result = normalize_port(port, scheme)

    assert result == expected



"""Tests for normalize_query function."""

import pytest

from url_normalize.url_normalize import normalize_query


@pytest.mark.parametrize(
    ("query", "expected"),
    [
        ("", ""),
        ("&&&", ""),
        ("param1=val1&param2=val2", "param1=val1&param2=val2"),
        ("Ç=Ç", "%C3%87=%C3%87"),
        ("%C3%87=%C3%87", "%C3%87=%C3%87"),
        ("q=C%CC%A7", "q=%C3%87"),
        ("q=%23test", "q=%23test"),  # Preserve encoded # in value, #31
        ("where=code%3D123", "where=code%3D123"),  # Preserve encoded = in value, #25
    ],
)
def test_normalize_query_result_is_expected(query, expected):
    """Assert we got expected results from the normalize_query function."""
    result = normalize_query(query)
    assert result == expected, query



"""Tests for normalize_fragment function."""

import pytest

from url_normalize.url_normalize import normalize_fragment


@pytest.mark.parametrize(
    ("fragment", "expected"),
    [
        ("", ""),
        ("fragment", "fragment"),
        ("пример", "%D0%BF%D1%80%D0%B8%D0%BC%D0%B5%D1%80"),
        ("!fragment", "%21fragment"),
        ("~fragment", "~fragment"),
        # Issue #36: Equal sign should not be encoded
        ("gid=1234", "gid=1234"),
    ],
)
def test_normalize_fragment_result_is_expected(fragment: str, expected: str) -> None:
    """Assert we got expected results from the normalize_fragment function."""
    result = normalize_fragment(fragment)
    assert result == expected, fragment



"""Deconstruct url tests."""

import pytest

from url_normalize.tools import URL, deconstruct_url


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        (
            "http://site.com",
            URL(
                fragment="",
                host="site.com",
                path="",
                port="",
                query="",
                scheme="http",
                userinfo="",
            ),
        ),
        (
            "http://user@www.example.com:8080/path/index.html?param=val#fragment",
            URL(
                fragment="fragment",
                host="www.example.com",
                path="/path/index.html",
                port="8080",
                query="param=val",
                scheme="http",
                userinfo="user@",
            ),
        ),
    ],
)
def test_deconstruct_url_result_is_expected(url: str, expected: URL) -> None:
    """Assert we got expected results from the deconstruct_url function."""
    result = deconstruct_url(url)
    assert result == expected, url



"""Tests for generic_url_cleanup function."""

from __future__ import annotations

import pytest

from url_normalize.url_normalize import generic_url_cleanup


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        ("//site/#!fragment", "//site/?_escaped_fragment_=fragment"),
        ("//site/page", "//site/page"),
        ("//site/?& ", "//site/"),
    ],
)
def test_generic_url_cleanup_result_is_expected(url: str, expected: str) -> None:
    """Assert we got expected results from the generic_url_cleanup function."""
    result = generic_url_cleanup(url)
    assert result == expected



"""Tests for normalize_scheme function."""

import pytest

from url_normalize.url_normalize import normalize_scheme


@pytest.mark.parametrize(
    ("scheme", "expected"),
    [
        ("http", "http"),
        ("HTTP", "http"),
    ],
)
def test_normalize_scheme_result_is_expected(scheme: str, expected: str) -> None:
    """Assert we got expected results from the normalize_scheme function."""
    result = normalize_scheme(scheme)
    assert result == expected, scheme



"""Tests for provide_url_scheme function."""

import pytest

from url_normalize.url_normalize import provide_url_scheme


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        ("", ""),
        ("-", "-"),
        ("/file/path", "/file/path"),
        ("//site/path", "https://site/path"),
        ("ftp://site/", "ftp://site/"),
        ("site/page", "https://site/page"),
    ],
)
def test_provide_url_scheme_result_is_expected(url: str, expected: str) -> None:
    """Assert we got expected results from the provide_url_scheme function."""
    result = provide_url_scheme(url)
    assert result == expected, url


def test_provide_url_scheme_accept_default_scheme_param() -> None:
    """Assert we could provide default_scheme param other than https."""
    url = "//site/path"
    expected = "http://site/path"

    actual = provide_url_scheme(url, default_scheme="http")

    assert actual == expected



"""Tests for provide_url_domain function."""

import pytest

from url_normalize.provide_url_domain import provide_url_domain


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        ("", ""),
        ("-", "-"),
        ("http://example.com/", "http://example.com/"),
        ("/file/path", "//example.com/file/path"),
        ("site/page", "site/page"),  # No change for relative paths
    ],
)
def test_provide_url_domain_result_is_expected(url: str, expected: str) -> None:
    """Assert we get expected results from provide_url_domain function."""
    result = provide_url_domain(url, default_domain="example.com")
    assert result == expected


def test_provide_url_domain_accept_different_domains():
    """Assert we could provide different default_domain values."""
    url = "/file/path"
    expected = "//custom-domain.org/file/path"

    actual = provide_url_domain(url, default_domain="custom-domain.org")

    assert actual == expected



"""Integrations tests."""

from __future__ import annotations

import pytest

from url_normalize import url_normalize


@pytest.mark.parametrize(
    "value",
    [
        "-",
        "",
        "/..foo",
        "/.foo",
        "/foo..",
        "/foo.",
        "ftp://user:pass@ftp.foo.net/foo/bar",
        "http://127.0.0.1/",
        "http://example.com:8080/",
        "http://example.com/?a&b",
        "http://example.com/?q=%5C",
        "http://example.com/?q=%C3%87",
        "http://example.com/?q=%E2%85%A0",
        "http://example.com/",
        "http://example.com/~jane",
        "http://example.com/a/b",
        "http://example.com/FOO",
        "http://user:password@example.com/",
        "http://www.foo.com:8000/foo",
        # from rfc2396bis
        "ftp://ftp.is.co.za/rfc/rfc1808.txt",
        "http://www.ietf.org/rfc/rfc2396.txt",
        "ldap://[2001:db8::7]/c=GB?objectClass?one",
        "mailto:John.Doe@example.com",
        "news:comp.infosystems.www.servers.unix",
        "tel:+1-816-555-1212",
        "telnet://192.0.2.16:80/",
        "urn:oasis:names:specification:docbook:dtd:xml:4.1.2",
        # Issue #36: Fragment with '=' should not be encoded
        "https://docs.google.com/spreadsheets/d/abcd/edit#gid=1234",
    ],
)
def test_url_normalize_no_changes_expected(value: str) -> None:
    """Assert url_normalize do not change URI if not required.

    http://www.intertwingly.net/wiki/pie/PaceCanonicalIds
    """
    assert url_normalize(value) == value


@pytest.mark.parametrize(
    ("value", "expected"),
    [
        ("/../foo", "/foo"),
        ("/./../foo", "/foo"),
        ("/./foo", "/foo"),
        ("/./foo/.", "/foo/"),
        ("//www.foo.com/", "https://www.foo.com/"),
        ("/foo/../bar", "/bar"),
        ("/foo/./bar", "/foo/bar"),
        ("/foo//", "/foo/"),
        ("/foo///bar//", "/foo/bar/"),
        ("/foo/bar/..", "/foo/"),
        ("/foo/bar/../..", "/"),
        ("/foo/bar/../../../../baz", "/baz"),
        ("/foo/bar/../../../baz", "/baz"),
        ("/foo/bar/../../", "/"),
        ("/foo/bar/../../baz", "/baz"),
        ("/foo/bar/../", "/foo/"),
        ("/foo/bar/../baz", "/foo/baz"),
        ("/foo/bar/.", "/foo/bar/"),
        ("/foo/bar/./", "/foo/bar/"),
        ("http://:@example.com/", "http://example.com/"),
        ("http://@example.com/", "http://example.com/"),
        ("http://127.0.0.1:80/", "http://127.0.0.1/"),
        ("http://example.com:081/", "http://example.com:81/"),
        ("http://example.com:80/", "http://example.com/"),
        ("http://example.com", "http://example.com/"),
        ("http://example.com/?b&a", "http://example.com/?b&a"),
        ("http://example.com/?q=%5c", "http://example.com/?q=%5C"),
        ("http://example.com/?q=%C7", "http://example.com/?q=%EF%BF%BD"),
        ("http://example.com/?q=C%CC%A7", "http://example.com/?q=%C3%87"),
        ("http://EXAMPLE.COM/", "http://example.com/"),
        ("http://example.com/%7Ejane", "http://example.com/~jane"),
        ("http://example.com/a/../a/b", "http://example.com/a/b"),
        ("http://example.com/a/./b", "http://example.com/a/b"),
        (
            "http://example.com/#!5753509/hello-world",
            "http://example.com/?_escaped_fragment_=5753509/hello-world",
        ),
        (
            "http://USER:pass@www.Example.COM/foo/bar",
            "http://USER:pass@www.example.com/foo/bar",
        ),
        ("http://www.example.com./", "http://www.example.com/"),
        ("http://www.foo.com:80/foo", "http://www.foo.com/foo"),
        ("http://www.foo.com.:81/foo", "http://www.foo.com:81/foo"),
        ("http://www.foo.com./foo/bar.html", "http://www.foo.com/foo/bar.html"),
        ("http://www.foo.com/%7Ebar", "http://www.foo.com/~bar"),
        ("http://www.foo.com/%7ebar", "http://www.foo.com/~bar"),
        (
            "пример.испытание/Служебная:Search/Test",
            "https://xn--e1afmkfd.xn--80akhbyknj4f/%D0%A1%D0%BB%D1%83%D0%B6%D0%B5%D0%B1%D0%BD%D0%B0%D1%8F:Search/Test",
        ),
        # Issue #19: http:example.com
        ("http:example.com", "http://example.com/"),
        ("http:example.com/path", "http://example.com/path"),
        ("ftp:test.com/files", "ftp://test.com/files"),
        ("https:www.example.com", "https://www.example.com/"),
    ],
)
def test_url_normalize_expected_changes(value: str, expected: str) -> None:
    """Assert url_normalize return expected results."""
    assert url_normalize(value) == expected


def test_url_normalize_filtered() -> None:
    """Assert url_normalize return expected results."""
    url = "/?a&b"
    expected = "/"

    actual = url_normalize(url, filter_params=True)

    assert actual == expected


def test_url_normalize_with_http_scheme() -> None:
    """Assert we could use http scheme as default."""
    url = "//www.foo.com/"
    expected = "http://www.foo.com/"

    actual = url_normalize(url, default_scheme="http")

    assert actual == expected


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        ("/foo.png", "https://example.com/foo.png"),
        ("//google.com", "https://google.com/"),
        ("//example.com:80/foo.png", "https://example.com:80/foo.png"),
        ("//example.com/foo.png?foo=bar", "https://example.com/foo.png?foo=bar"),
        ("http://google.com", "http://google.com/"),
    ],
)
def test_url_normalize_with_default_domain(url: str, expected: str) -> None:
    """Assert we could use default_domain parameter."""
    actual = url_normalize(url, default_domain="example.com")

    assert actual == expected


def test_url_normalize_with_default_domain_and_scheme() -> None:
    """Assert we can use both default_domain and default_scheme together."""
    url = "/foo.png"
    expected = "http://example.com/foo.png"

    actual = url_normalize(url, default_scheme="http", default_domain="example.com")

    assert actual == expected



"""Tests for the command line interface."""

import subprocess
import sys
from unittest.mock import patch

import pytest

from url_normalize import __version__
from url_normalize.cli import main


def run_cli(*args: str) -> subprocess.CompletedProcess:
    """Run the CLI command with given arguments.

    Params:
        *args: Command line arguments to pass to the CLI.

    Returns:
        A completed process with stdout, stderr, and return code.

    """
    command = [sys.executable, "-m", "url_normalize.cli", *list(args)]
    return subprocess.run(  # noqa: S603
        command, capture_output=True, text=True, check=False
    )


def test_cli_error_handling(capsys, monkeypatch):
    """Test CLI error handling when URL normalization fails."""
    with patch("url_normalize.cli.url_normalize") as mock_normalize:
        mock_normalize.side_effect = Exception("Simulated error")
        monkeypatch.setattr("sys.argv", ["url-normalize", "http://example.com"])

        with pytest.raises(SystemExit) as excinfo:
            main()

        assert excinfo.value.code == 1
        captured = capsys.readouterr()
        assert "Error normalizing URL: Simulated error" in captured.err
        assert not captured.out


def test_cli_basic_normalization() -> None:
    """Test basic URL normalization via CLI."""
    url = "http://EXAMPLE.com/./path/../other/"
    expected = "http://example.com/other/"

    result = run_cli(url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_basic_normalization_short_args() -> None:
    """Test basic URL normalization via CLI using short arguments."""
    url = "http://EXAMPLE.com/./path/../other/"
    expected = "http://example.com/other/"
    # Using short args where applicable (none for the URL itself)

    result = run_cli(url)  # No short args needed for basic case

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_scheme() -> None:
    """Test default scheme addition via CLI."""
    url = "//example.com"
    expected = "https://example.com/"

    result = run_cli(url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_scheme_short_arg() -> None:
    """Test default scheme addition via CLI using short argument."""
    url = "//example.com"
    expected = "https://example.com/"

    result = run_cli(url)  # Default scheme is implicit, no arg needed

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_custom_default_scheme() -> None:
    """Test custom default scheme via CLI."""
    url = "//example.com"
    expected = "ftp://example.com/"

    result = run_cli("--default-scheme", "ftp", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_custom_default_scheme_short_arg() -> None:
    """Test custom default scheme via CLI using short argument."""
    url = "//example.com"
    expected = "ftp://example.com/"

    result = run_cli("-s", "ftp", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_filter_params() -> None:
    """Test parameter filtering via CLI."""
    url = "http://google.com?utm_source=test&q=1"
    expected = "http://google.com/?q=1"

    result = run_cli("--filter-params", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_filter_params_short_arg() -> None:
    """Test parameter filtering via CLI using short argument."""
    url = "http://google.com?utm_source=test&q=1"
    expected = "http://google.com/?q=1"

    result = run_cli("-f", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_param_allowlist() -> None:
    """Test parameter allowlist via CLI."""
    url = "http://example.com?remove=me&keep=this&remove_too=true"
    expected = "http://example.com/?keep=this"
    # Use filter_params to enable filtering, then allowlist to keep specific ones

    result = run_cli("-f", "-p", "keep", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_param_allowlist_multiple() -> None:
    """Test parameter allowlist with multiple params via CLI."""
    url = "http://example.com?remove=me&keep=this&keep_too=yes&remove_too=true"
    expected = "http://example.com/?keep=this&keep_too=yes"

    result = run_cli("-f", "-p", "keep,keep_too", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_param_allowlist_without_filtering() -> None:
    """Test allowlist has no effect if filtering is not enabled."""
    url = "http://example.com?remove=me&keep=this&remove_too=true"
    expected = "http://example.com/?remove=me&keep=this&remove_too=true"
    # Not using -f, so allowlist should be ignored

    result = run_cli("-p", "keep", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_no_url() -> None:
    """Test CLI error when no URL is provided."""
    result = run_cli()

    assert result.returncode != 0
    assert "the following arguments are required: url" in result.stderr


def test_cli_version_long() -> None:
    """Test version output with --version flag."""
    result = run_cli("--version")

    assert result.returncode == 0
    assert __version__ in result.stdout
    assert not result.stderr


def test_cli_version_short() -> None:
    """Test version output with -v flag."""
    result = run_cli("-v")

    assert result.returncode == 0
    assert __version__ in result.stdout
    assert not result.stderr


@pytest.mark.skipif(
    sys.platform == "win32", reason="Charset handling differs on Windows CLI"
)
def test_cli_charset() -> None:
    """Test charset handling via CLI (might be platform-dependent)."""
    # Example using Cyrillic characters which need correct encoding
    url = "http://пример.рф/path"
    expected_idn = "http://xn--e1afmkfd.xn--p1ai/path"

    # Test with default UTF-8
    result_utf8 = run_cli(url)

    assert result_utf8.returncode == 0
    assert result_utf8.stdout.strip() == expected_idn
    assert not result_utf8.stderr

    # Test specifying UTF-8 explicitly
    result_charset = run_cli("--charset", "utf-8", url)

    assert result_charset.returncode == 0
    assert result_charset.stdout.strip() == expected_idn
    assert not result_charset.stderr

    # Test specifying UTF-8 explicitly using short arg
    result_charset_short = run_cli("-c", "utf-8", url)

    assert result_charset_short.returncode == 0
    assert result_charset_short.stdout.strip() == expected_idn
    assert not result_charset_short.stderr


def test_cli_default_domain() -> None:
    """Test adding default domain to absolute path via CLI."""
    url = "/path/to/image.png"
    expected = "https://example.com/path/to/image.png"

    result = run_cli("--default-domain", "example.com", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_domain_short_arg() -> None:
    """Test adding default domain using short argument."""
    url = "/path/to/image.png"
    expected = "https://example.com/path/to/image.png"

    result = run_cli("-d", "example.com", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_domain_with_scheme() -> None:
    """Test adding default domain with custom scheme."""
    url = "/path/to/image.png"
    expected = "http://example.com/path/to/image.png"

    result = run_cli("-d", "example.com", "-s", "http", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_domain_no_effect_on_absolute_urls() -> None:
    """Test default domain has no effect on absolute URLs."""
    url = "http://original-domain.com/path"
    expected = "http://original-domain.com/path"

    result = run_cli("-d", "example.com", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr


def test_cli_default_domain_no_effect_on_relative_paths() -> None:
    """Test default domain has no effect on relative paths."""
    url = "path/to/file.html"
    # This becomes a regular URL with the default scheme
    expected = "https://path/to/file.html"

    result = run_cli("-d", "example.com", url)

    assert result.returncode == 0
    assert result.stdout.strip() == expected
    assert not result.stderr



"""Tests for normalize_path function."""

import pytest

from url_normalize.url_normalize import normalize_path


@pytest.mark.parametrize(
    ("path", "expected"),
    [
        ("..", "/"),
        ("", "/"),
        ("/../foo", "/foo"),
        ("/..foo", "/..foo"),
        ("/./../foo", "/foo"),
        ("/./foo", "/foo"),
        ("/./foo/.", "/foo/"),
        ("/.foo", "/.foo"),
        ("/", "/"),
        ("/foo..", "/foo.."),
        ("/foo.", "/foo."),
        ("/FOO", "/FOO"),
        ("/foo/../bar", "/bar"),
        ("/foo/./bar", "/foo/bar"),
        ("/foo//", "/foo/"),
        ("/foo///bar//", "/foo/bar/"),
        ("/foo/bar/..", "/foo/"),
        ("/foo/bar/../..", "/"),
        ("/foo/bar/../../../../baz", "/baz"),
        ("/foo/bar/../../../baz", "/baz"),
        ("/foo/bar/../../", "/"),
        ("/foo/bar/../../baz", "/baz"),
        ("/foo/bar/../", "/foo/"),
        ("/foo/bar/../baz", "/foo/baz"),
        ("/foo/bar/.", "/foo/bar/"),
        ("/foo/bar/./", "/foo/bar/"),
        # Issue #25: we should preserve ? in the path
        ("/More+Tea+Vicar%3F/discussion", "/More+Tea+Vicar%3F/discussion"),
    ],
)
def test_normalize_path_result_is_expected(path: str, expected: str) -> None:
    """Assert we got expected results from the normalize_path function."""
    result = normalize_path(path, "http")
    assert result == expected, path



"""Reconstruct url tests."""

from __future__ import annotations

import pytest

from url_normalize.tools import URL, reconstruct_url


@pytest.mark.parametrize(
    ("url_obj", "expected"),
    [
        (
            URL(
                fragment="",
                host="site.com",
                path="",
                port="",
                query="",
                scheme="http",
                userinfo="",
            ),
            "http://site.com",
        ),
        (
            URL(
                fragment="fragment",
                host="www.example.com",
                path="/path/index.html",
                port="8080",
                query="param=val",
                scheme="http",
                userinfo="user@",
            ),
            "http://user@www.example.com:8080/path/index.html?param=val#fragment",
        ),
    ],
)
def test_reconstruct_url_result_is_expected(url_obj: URL, expected: str) -> None:
    """Assert we got expected results from the reconstruct_url function."""
    result = reconstruct_url(url_obj)
    assert result == expected, url_obj



"""URL parameter filtering test module."""

from __future__ import annotations

import pytest

from url_normalize import url_normalize


def test_param_filtering_disabled_by_default():
    """Test that parameter filtering is disabled by default."""
    url = "https://www.google.com/search?q=test&utm_source=test"
    assert url_normalize(url) == url


def test_empty_query():
    """Test handling empty query strings."""
    assert url_normalize("https://example.com/page?") == "https://example.com/page"


def test_custom_allowlist():
    """Test custom allowlist functionality with preserved order."""
    custom_allowlist = {"example.com": ["page", "id"], "google.com": ["q", "lang"]}

    # Order should match input query string order
    assert (
        url_normalize(
            "https://example.com/search?page=1&id=123&utm_source=test",
            filter_params=True,
            param_allowlist=custom_allowlist,
        )
        == "https://example.com/search?page=1&id=123"
    )

    assert (
        url_normalize(
            "https://google.com/search?q=test&ie=utf8&lang=en",
            filter_params=True,
            param_allowlist=custom_allowlist,
        )
        == "https://google.com/search?q=test&lang=en"
    )


def test_custom_list_allowlist():
    """Test custom list allowlist functionality."""
    assert (
        url_normalize(
            "https://google.com/search?qq=test&ie=utf8&utm_source=test",
            filter_params=True,
            param_allowlist=["ie", "qq"],
        )
        == "https://google.com/search?qq=test&ie=utf8"
    )


@pytest.mark.parametrize(
    ("url", "expected"),
    [
        # Basic parameter filtering
        (
            "https://www.google.com/search?q=test&utm_source=test",
            "https://www.google.com/search?q=test",
        ),
        (
            "https://www.youtube.com/watch?v=12345&utm_source=share",
            "https://www.youtube.com/watch?v=12345",
        ),
        # With www subdomain
        (
            "https://www.google.com/search?q=test&ref=test",
            "https://www.google.com/search?q=test",
        ),
        # With port number
        (
            "https://google.com:8080/search?q=test&ref=test",
            "https://google.com:8080/search?q=test",
        ),
        # Default allowlist cases
        (
            "https://www.google.com/search?q=test&utm_source=test&ie=utf8",
            "https://www.google.com/search?q=test&ie=utf8",
        ),
        (
            "https://www.baidu.com/s?wd=test&utm_source=test&ie=utf8",
            "https://www.baidu.com/s?wd=test&ie=utf8",
        ),
        (
            "https://youtube.com/watch?v=12345&utm_source=test&search_query=test",
            "https://youtube.com/watch?v=12345&search_query=test",
        ),
        # Non-allowlisted domain
        ("https://example.org/page?a=1&b=2", "https://example.org/page"),
    ],
)
def test_parameter_filtering(url: str, expected: str):
    """Test URL parameter filtering functionality with various scenarios."""
    assert url_normalize(url, filter_params=True) == expected
